import dataclasses
import os
import platform
from typing import Callable, Union


STATIC_PATH = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    'static'
)


@dataclasses.dataclass
class Resource:
    name: str
    content: str


def load_resource(name: str) -> Resource:
    with open(os.path.join(STATIC_PATH, name), 'r') as f:
        return Resource(name, f.read())


class Term:
    if platform.system() == 'Windows':
        os.system('color')

    BLACK = "\033[30m"
    R = "\033[31m"
    G = "\033[32m"
    BG = "\033[1;32m"
    Y = "\033[33m"
    B = "\033[34m"
    M = "\033[35m"
    C = "\033[36m"
    W = "\033[37m"
    END = "\033[0;0m"
    NORM = "\033[1m"
    BOLD = "\033[1m"

    @staticmethod
    def dev(*values: object, end: str = '\n', sep: str = ' ') -> None:
        print(f'{Term.M}[dev]{Term.END}', *values, end=end, sep=sep)

    @staticmethod
    def info(*values: object, end: str = '\n', sep: str = ' ') -> None:
        print(f'{Term.C}[info]{Term.END}', *values, end=end, sep=sep)

    @staticmethod
    def warn(*values: object, end: str = '\n', sep: str = ' ') -> None:
        print(f'{Term.Y}[warn]{Term.END}', *values, end=end, sep=sep)

    @staticmethod
    def error(*values: object, end: str = '\n', sep: str = ' ') -> None:
        print(f'{Term.R}[error]{Term.END}', *values, end=end, sep=sep)

    @staticmethod
    def confirm(message: str) -> bool:
        response = input(
            f'{Term.C}{message}{Term.END} [y/N] ').strip().lower()
        while True:
            if not response or response.startswith('n'):
                return False
            if response.startswith('y'):
                return True
            print(f'{Term.R}Invalid response{Term.END}')
            response = input(
                f'{Term.C}{message}{Term.END} [y/N] ').strip().lower()

    @staticmethod
    def ask(message: str, validator: Union[None, Callable[[str], bool]] = None) -> str:
        while True:
            response = input(message).strip()
            if not validator or validator(response):
                return response

            Term.info(f'Invalid response: {response}')

    @staticmethod
    def ask_file(message: str, base_dir: Union[str, None] = None) -> str:
        file = input(message).strip()

        while True:
            if not file:
                print(f'Please enter a non-empty file path')
                file = input(message).strip()
                continue

            full_path = f'{base_dir}/{file}' if base_dir else file
            if os.path.isfile(full_path):
                return file

            Term.error(f'File not found: {full_path}')
            file = input(message).strip()

    @staticmethod
    def ask_dir(message: str, base_dir: Union[str, None] = None) -> str:
        dir = input(message).strip()

        while True:
            if not dir:
                print(f'Please enter a non-empty dir path')
                dir = input(message).strip()
                continue

            full_path = f'{base_dir}/{dir}' if base_dir else dir
            if os.path.isdir(full_path):
                return dir

            Term.error(f'\'{full_path}\' is not a dir')
            dir = input(message).strip()



import argparse
import dataclasses
import os
import shlex
import subprocess
from typing import Union

from flask_livetw.util import Term


DEFAULT_STATIC_FOLDER = 'static'

DEFAULT_TEMPLATE_FOLDER = 'templates'
DEFAULT_TEMPLATE_GLOB = '**/*.html'
DEFAULT_TEMPLATE_ROOT_LAYOUT = 'layout.html'

DEFAULT_LIVE_RELOAD_FILE = '.dev/live_reload.js'

DEFAULT_TWCSS_FILE = '.dev/tailwindcss.css'

DEFAULT_MINIFIED_TWCSS_FILE = 'tailwindcss_min.css'

DEFAULT_UPDATE_GITIGNORE = False


@dataclasses.dataclass
class Config:
    gitignore: bool

    flask_root: Union[str, None]

    static_folder: str

    templates_folder: str
    templates_glob: str

    root_layout_file: str
    live_reload_file: str
    twcss_file: str
    minified_twcss_file: str


def create_cli() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description='Mods a Flask app to use TailwindCSS in a dev server like manner.',
        allow_abbrev=True
    )

    parser.add_argument(
        '-Y', '--yes', dest='all_yes', action='store_true', default=False,
        help='Answer yes to all questions.'
    )

    parser.add_argument(
        '-D', '--default', dest='default', action='store_true', default=False,
        help='Use default values for all options.'
    )

    parser.add_argument(
        '--gi', '--gitignore', dest='gitignore', action='store_true',
        help=f'Update .gitignore to exclude dev mode related files. Default: {DEFAULT_UPDATE_GITIGNORE}'
    )

    parser.add_argument(
        '--fr', '--flask-root', dest='flask_root', type=str,
        help=f'Flask app root path (relative to cwd).'
    )

    parser.add_argument(
        '--sf', '--static-folder', dest='static_folder', type=str,
        help=f'Static folder path. Default: <your project>/{DEFAULT_STATIC_FOLDER}'
    )

    parser.add_argument(
        '--tf', '--templates-folder', dest='templates_folder', type=str,
        help=f'Templates folder path. Default: <your project>/{DEFAULT_TEMPLATE_FOLDER}'
    )
    parser.add_argument(
        '--tg', '--templates-glob', dest='templates_glob', type=str,
        help=f'Templates glob pattern. Default: {DEFAULT_TEMPLATE_GLOB}'
    )
    parser.add_argument(
        '--rlf', '--root-layout-file', dest='root_layout_file', type=str,
        help=f'Template root layout file. Default: <templates folder>/{DEFAULT_TEMPLATE_ROOT_LAYOUT}'
    )

    parser.add_argument(
        '--lrf', '--live-reload-file', dest='live_reload_file', type=str,
        help=f'Live reload js file, relative to static folder. Default: <static folder>/{DEFAULT_LIVE_RELOAD_FILE}'
    )
    parser.add_argument(
        '--twf', '--twcss-file', dest='twcss_file', type=str,
        help=f'Generated css file, relative to static folder. Default: <static folder>/{DEFAULT_TWCSS_FILE}'
    )
    parser.add_argument(
        '--mtwf', '--minified-twcss-file', dest='minified_twcss_file', type=str,
        help=f'Minified css file, relative to static folder. Default: <static folder>/{DEFAULT_MINIFIED_TWCSS_FILE}'
    )

    return parser


def check_requirements() -> int:
    cwd = os.getcwd()
    print(
        f'The modding will continue on the current working directory:\n> {Term.BOLD}{cwd}{Term.END} ')
    continue_install = Term.confirm("Continue?")

    if not continue_install:
        Term.dev("modding canceled")
        return 1

    python_cmd = shlex.split("python --version")
    python_cmd_result = subprocess.run(
        python_cmd, shell=True, check=True, capture_output=True
    )

    if python_cmd_result.returncode != 0:
        Term.error('python --version failed, terminating script')
        return python_cmd_result.returncode

    version = python_cmd_result.stdout.decode('utf-8')
    if version != 'Python 3.8.10':
        Term.warn("Current python version is 3.8.10")
        continue_install = Term.confirm("Continue?")
        if not continue_install:
            Term.dev("modding canceled")
            return 1

    return 0


def get_config(args: argparse.Namespace) -> Config:
    if args.default:
        return Config(
            gitignore=DEFAULT_UPDATE_GITIGNORE,

            flask_root=None,

            static_folder=DEFAULT_STATIC_FOLDER,

            templates_folder=DEFAULT_TEMPLATE_FOLDER,
            templates_glob=DEFAULT_TEMPLATE_GLOB,

            root_layout_file=DEFAULT_TEMPLATE_ROOT_LAYOUT,
            live_reload_file=DEFAULT_LIVE_RELOAD_FILE,
            twcss_file=DEFAULT_TWCSS_FILE,
            minified_twcss_file=DEFAULT_MINIFIED_TWCSS_FILE
        )

    gitignore = True if args.gitignore else False

    flask_root = args.flask_root
    if flask_root is None:
        flask_root = Term.ask_dir('Flask app root path (relative to cwd): ')

    static_folder = args.static_folder
    if static_folder is None:
        static_folder = Term.ask_dir(
            f'Static folder (relative to \'cwd/{flask_root}\'): ',
            flask_root
        )

    templates_folder = args.templates_folder
    if templates_folder is None:
        templates_folder = Term.ask_dir(
            f'Templates folder (relative to \'cwd/{flask_root}\'): ',
            flask_root
        )

    templates_glob = args.templates_glob
    if templates_glob is None:
        templates_glob = Term.ask(
            f'Templates glob pattern (default: {templates_folder}/{DEFAULT_TEMPLATE_GLOB}): ',
            lambda glob: not glob
        )

    root_layout_file = args.root_layout_file
    if root_layout_file is None:
        root_layout_file = Term.ask(
            f'Template root layout file (default: {templates_folder}/{DEFAULT_TEMPLATE_ROOT_LAYOUT}): ',
            lambda glob: not glob
        )

    live_reload_file = args.live_reload_file
    if live_reload_file is None:
        live_reload_file = Term.ask(
            f'Live reload js file (default: {static_folder}/{DEFAULT_LIVE_RELOAD_FILE}): ',
            lambda glob: not glob
        )

    twcss_file = args.twcss_file
    if twcss_file is None:
        twcss_file = Term.ask(
            f'Generated css file (default: {static_folder}/{DEFAULT_TWCSS_FILE}): ',
            lambda glob: not glob
        )

    minified_twcss_file = args.minified_twcss_file
    if minified_twcss_file is None:
        minified_twcss_file = Term.ask(
            f'Minified css file (default: {static_folder}/{DEFAULT_MINIFIED_TWCSS_FILE}): ',
            lambda glob: not glob
        )

    full_static_folder = f'{flask_root}/{static_folder}'
    full_templates_folder = f'{flask_root}/{templates_folder}'

    return Config(
        gitignore=gitignore,

        flask_root=flask_root,

        static_folder=full_static_folder,

        templates_folder=full_templates_folder,
        templates_glob=f'{full_templates_folder}/{templates_glob}',

        root_layout_file=f'{full_templates_folder}/{root_layout_file}',
        live_reload_file=live_reload_file,
        twcss_file=twcss_file,
        minified_twcss_file=minified_twcss_file,
    )



#!/usr/bin/env python

import websockets.server as ws_server
import websockets.legacy.protocol as ws_protocol
import asyncio
import datetime
import json
import subprocess
import os
import dotenv
import concurrent.futures
import platform
import argparse
import shlex


# docs:
# - https://websockets.readthedocs.io/en/stable/intro/tutorial2.html
# - https://websockets.readthedocs.io/en/stable/reference/asyncio/server.html

class Term:
    if platform.system() == 'Windows':
        os.system('color')

    BLACK = "\033[30m"
    R = "\033[31m"
    G = "\033[32m"
    BG = "\033[1;32m"
    Y = "\033[33m"
    B = "\033[34m"
    M = "\033[35m"
    C = "\033[36m"
    W = "\033[37m"
    END = "\033[0;0m"
    NORMAL = "\033[1m"
    BOLD = "\033[1m"


def dev_print(*values: object):
    print(f'{Term.M}[dev]{Term.END}', *values)


def int_or_default(value: 'str|None', default: int) -> int:
    if value is None:
        return default
    try:
        return int(value)
    except ValueError:
        return default


dotenv.load_dotenv()

LRWS_HOST = os.getenv('LIVE_RELOAD_WS_HOST', '127.0.0.1')
LRWS_PORT = int_or_default(os.getenv('LIVE_RELOAD_WS_PORT'), 5678)
TW_WATCH_PATH = os.getenv('TW_WATCH_PATH')
TW_OUTPUT_PATH = os.getenv(
    'TW_OUTPUT_PATH', '{tailwind_output_placeholder}'
)
TW_OUTPUT_PATH_BUILD = os.getenv(
    'TW_OUTPUT_PATH_BUILD', '{minified_tailwind_output_placeholder}'
)


LR_CONNECTIONS: 'set[ws_server.WebSocketServerProtocol]' = set()


async def handle_connection(websocket: ws_server.WebSocketServerProtocol):
    LR_CONNECTIONS.add(websocket)
    try:
        await websocket.wait_closed()
    finally:
        LR_CONNECTIONS.remove(websocket)


async def live_reload_server(host: str, port: int):
    async with ws_server.serve(handle_connection, host, port) as server:
        dev_print(
            f'Live reload {Term.G}ready{Term.END} on {Term.C}ws://{host}:{Term.BOLD}{port}{Term.END}')

        await server.wait_closed()

        dev_print(f'Live reload {Term.G}closed{Term.END}')


def handle_tw_output(process: 'subprocess.Popen[bytes]'):
    if process.stdout is None:
        return

    for line in iter(process.stdout.readline, b''):
        if process.poll() is not None:
            break

        if (line.startswith(b'Done')):
            ws_protocol.broadcast(LR_CONNECTIONS, json.dumps({
                "type": "TRIGGER_FULL_RELOAD",
                "data": datetime.datetime.now().isoformat()
            }))

        print(f'{Term.C}[twcss]{Term.END} {line.decode("utf-8")}', end='')


def handle_flask_output(process: 'subprocess.Popen[bytes]'):
    if process.stdout is None:
        return

    for line in iter(process.stdout.readline, b''):
        if process.poll() is not None:
            break

        print(f'{Term.G}[flask]{Term.END} {line.decode("utf-8")}', end='')


async def dev_server(cli: argparse.Namespace):
    def live_reload_coroutine():
        if cli.no_live_reload or cli.no_tailwind:
            return None

        host = LRWS_HOST if cli.live_reload_host is None else cli.live_reload_host
        port = LRWS_PORT if cli.live_reload_port is None else cli.live_reload_port

        return live_reload_server(host, port)

    def tw_cli_executor(loop: asyncio.AbstractEventLoop, pool: concurrent.futures.ThreadPoolExecutor):
        if cli.no_tailwind:
            return None

        input_arg = f''
        if cli.tailwind_input is not None:
            input_arg = f'-i {cli.tailwind_input}'
        elif TW_WATCH_PATH is not None:
            input_arg = f'-i {TW_WATCH_PATH}'

        output_arg = f'-o {TW_OUTPUT_PATH}'
        if cli.tailwind_output is not None:
            output_arg = f'-o {cli.tailwind_output}'

        minify_arg = '--minify' if cli.tailwind_minify else ''

        cmd = f'tailwindcss --watch {input_arg} {output_arg} {minify_arg}'

        process = subprocess.Popen(
            shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        return loop.run_in_executor(pool, handle_tw_output, process)

    def flask_server_executor(loop: asyncio.AbstractEventLoop, pool: concurrent.futures.ThreadPoolExecutor):
        if cli.no_flask:
            return None

        host_arg = '--host' + cli.flask_host if cli.flask_host is not None else ''

        port_arg = '--port' + cli.flask_port if cli.flask_port is not None else ''

        debug_arg = '--debug' if cli.flask_mode == 'debug' else ''

        exclude_patterns = ['*/**/dev.py', '*/**/install_dev_mode.py']
        if cli.flask_exclude_patterns is not None:
            exclude_patterns.extend(cli.flask_exclude_patterns)
        exclude_patterns_arg = f'--exclude-patterns {";".join(exclude_patterns)}'

        cmd = f'flask run {host_arg} {port_arg} {debug_arg} {exclude_patterns_arg}'

        process = subprocess.Popen(
            shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        return loop.run_in_executor(pool, handle_flask_output, process)

    loop = asyncio.get_running_loop()

    with concurrent.futures.ThreadPoolExecutor(max_workers=3) as pool:
        maybe_future_like = (
            live_reload_coroutine(),
            tw_cli_executor(loop, pool),
            flask_server_executor(loop, pool)
        )

        futures = (
            future
            for future in maybe_future_like
            if future is not None
        )

        _ = await asyncio.gather(*futures, return_exceptions=True)


def minify_tailwindcss(cli: argparse.Namespace):
    input_arg = f''
    if cli.input is not None:
        input_arg = f'-i {cli.input}'

    output_arg = f'-o {TW_OUTPUT_PATH_BUILD}'
    if cli.output is not None:
        output_arg = f'-o {cli.output}'

    minify_arg = '--minify' if cli.minify else ''

    command = f'tailwindcss {input_arg} {output_arg} {minify_arg}'

    dev_print(f'Minifying tailwindcss for production...')

    build_result = subprocess.run(shlex.split(command))

    if build_result.returncode != 0:
        dev_print(f'Tailwind build for production {Term.R}fail{Term.END}')
        return build_result.returncode

    dev_print(f'Tailwind build for production {Term.G}ready{Term.END}')
    return build_result.returncode


def main():
    cli_args = cli().parse_args()

    if cli_args.command == 'build':
        return minify_tailwindcss(cli_args)

    if cli_args.command == 'dev':
        asyncio.run(dev_server(cli_args))
        return 0

    dev_print(f'Unknown command {Term.BOLD}{cli_args.command}{Term.END}')
    dev_print(f'Use {Term.BOLD}dev.py [-h | --help]{Term.END} for help.')
    return 1


def cli() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description='Enhanced dev environment for flask apps.',
        allow_abbrev=True
    )

    subparsers = parser.add_subparsers(
        title='commands', dest='command',
        help='availible commands',
        required=True
    )

    parser_build = subparsers.add_parser(
        name='build',
        description='Build the tailwindcss of the provided input as a single css file.',
        help='Build tailwindcss for production.',
        allow_abbrev=True
    )
    parser_build.add_argument(
        '-i', '--input', dest='input', type=str,
        help='Input path, accepts glob patterns.'
    )
    parser_build.add_argument(
        '-o', '--output', dest='output', type=str,
        help='Output path.'
    )
    build_minify_group = parser_build.add_mutually_exclusive_group()
    build_minify_group.add_argument(
        '--minify', dest='minify', action='store_true',
        help='Minify output.'
    )
    build_minify_group.add_argument(
        '--no-minify', dest='minify', action='store_false',
        help='Do not minify output.'
    )
    parser_build.set_defaults(minify=True)

    parser_dev = subparsers.add_parser(
        name='dev',
        description='''\
Extended dev mode for flask apps.
By default runs the flask app in debug mode,
tailwindcss in watch mode and live reload server.''',
        help='Run a development server.',
        allow_abbrev=True
    )
    parser_dev.add_argument(
        '--no-live-reload', dest='no_live_reload', action='store_true', default=False,
        help='Disable live reload server.'
    )
    parser_dev.add_argument(
        '-lrh', '--live-reload-host', dest='live_reload_host', type=str,
        help='Hostname for live reload server.'
    )
    parser_dev.add_argument(
        '-lrp', '--live-reload-port', dest='live_reload_port', type=int,
        help='Port for live reload server.'
    )

    parser_dev.add_argument(
        '--no-flask', dest='no_flask', action='store_true', default=False,
        help='Disable flask server.'
    )
    parser_dev.add_argument(
        '-fh', '--flask-host', dest='flask_host', type=str,
        help='Hostname for flask server.'
    )
    parser_dev.add_argument(
        '-fp', '--flask-port', dest='flask_port', type=int,
        help='Port for flask server.'
    )
    parser_dev.add_argument(
        '-fm', '--flask-mode',  dest='flask_mode', choices=('debug', 'no-debug'), default='debug',
        help='If debug mode is enabled, the flask server will be started with --debug flag. Default: debug.'
    )
    parser_dev.add_argument(
        '--flask-exclude-patterns', dest='flask_exclude_patterns', type=str, nargs='+',
        help='File exclude patterns for flask server. Base: */**/dev.py */**/install_dev_mode.py'
    )

    parser_dev.add_argument(
        '-nt', '--no-tailwind', dest='no_tailwind', action='store_true', default=False,
        help='Disable tailwindcss generation. If tailwindcss is disabled the live reload server will not be started.'
    )
    parser_dev.add_argument(
        '-ti', '--tailwind-input', dest='tailwind_input', type=str,
        help='Input path to watch for changes. Includes glob patterns.'
    )
    parser_dev.add_argument(
        '-to', '--tailwind-output', dest='tailwind_output', type=str,
        help='Output path for the generated css file.'
    )
    parser_dev.add_argument(
        '-tm', '--tailwind-minify', dest='tailwind_minify', action='store_true', default=False,
        help='Enables minification of the generated css file.'
    )

    return parser


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python

import os
import shlex
import subprocess

from flask_livetw import cli
from flask_livetw.util import Term, load_resource


DEV_DEPENDENCIES = 'pytailwindcss websockets python-dotenv'


LIVE_RELOAD_SCRIPT = load_resource('live_reload.js')
DEV_SCRIPT = load_resource('dev.py')
TAILWIND_CONFIG = load_resource('tailwind.config.js')
LAYOUT_TEMPLATE = load_resource('layout.html')


def generate_tw_config(content_glob: str) -> str:
    return TAILWIND_CONFIG.content.replace(
        '{content_glob_placeholder}',
        content_glob
    )


def generate_dev_script(twcss_file: str, minified_twcss_file: str) -> str:
    return DEV_SCRIPT.content \
        .replace(
            '{tailwind_output_placeholder}',
            twcss_file
        ).replace(
            '{minified_tailwind_output_placeholder}',
            minified_twcss_file
        )


def generate_live_reload_template(live_reload_file: str, twcss_file: str, minified_twcss_file: str) -> str:
    return ('''
  {% if config.DEBUG %}
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename=\'''' + twcss_file + '''\') }}">
    <script src="{{ url_for('static', filename=\'''' + live_reload_file + '''\') }}" defer></script>
  {% else %}
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename=\'''' + minified_twcss_file + '''\') }}">
  {% endif %}
''').strip('\n')


def generate_layout_template(live_reload_file: str, twcss_file: str, minified_twcss_file: str) -> str:
    return LAYOUT_TEMPLATE.content.replace(
        '{live_reload_template_placeholder}',
        generate_live_reload_template(
            live_reload_file, twcss_file, minified_twcss_file)
    )


def install_dev_dependencies() -> None:
    Term.dev('Installing required dev dependencies...')

    poetry_cmd = shlex.split(f"poetry add --group=dev {DEV_DEPENDENCIES}")

    try:
        result = subprocess.run(poetry_cmd, shell=True, check=True)
    except subprocess.CalledProcessError as e:
        Term.error(e)
        Term.info('Dev dependencies installation failed, terminating script')
        exit(1)

    if result.returncode != 0:
        Term.info('Dev dependencies installation failed, terminating script')
        exit(result.returncode)

    Term.dev('Dev dependencies installation complete')


def init_tailwindcss(content_glob: str) -> None:
    Term.dev('Initializing tailwindcss...')

    tailwind_init = "tailwindcss init"

    try:
        result = subprocess.run(tailwind_init, shell=True, check=True)
    except subprocess.CalledProcessError as e:
        Term.error(e)
        Term.info('Tailwindcss initialization failed, terminating script')
        exit(1)

    if result.returncode != 0:
        Term.info('Tailwindcss initialization failed, terminating script')
        exit(result.returncode)

    with open('tailwind.config.js', 'w') as f:
        f.write(generate_tw_config(content_glob))

    Term.dev('Tailwindcss initialization complete')


def generate_files(live_reload_file: str, twcss_file: str, minified_twcss_file: str) -> None:
    with open(DEV_SCRIPT.name, 'w') as f:
        f.write(generate_dev_script(twcss_file, minified_twcss_file))

    try:
        with open(live_reload_file, 'w') as f:
            f.write(LIVE_RELOAD_SCRIPT.content)

    except FileNotFoundError:
        os.makedirs(
            os.path.dirname(live_reload_file),
            exist_ok=True
        )
        with open(live_reload_file, 'w') as f:
            f.write(LIVE_RELOAD_SCRIPT.content)


def update_layout(root_layout_template: str, live_reload_file: str, twcss_file: str, minified_twcss_file: str) -> int:
    root_layout = root_layout_template

    try:
        with open(root_layout, '+r') as f:
            layout = f.read()
            if '</head>' not in layout:
                Term.error(
                    'Root layour is malformed, the </head> tag is missing. Please check your root layout file.')
                return 1

            layout = layout.replace(
                '</head>',
                generate_live_reload_template(
                    live_reload_file,
                    twcss_file,
                    minified_twcss_file
                ) + '</head>'
            )
            f.seek(0)
            f.write(layout)
            f.truncate()
            return 0
    except FileNotFoundError as e:
        Term.warn(e)
        os.makedirs(
            os.path.dirname(root_layout),
            exist_ok=True
        )
        with open(root_layout, 'w') as f:
            f.write(generate_layout_template(
                live_reload_file,
                twcss_file,
                minified_twcss_file
            ))

    return 0


def update_gitignore(static_folder: str, twcss_file: str) -> None:
    content = f'''
# flask-live-twcss
{static_folder}/{twcss_file}
'''

    try:
        with open('.gitignore', 'a') as f:
            f.write(content)
    except FileNotFoundError:
        Term.info('Missing .gitignore file, creating one...')
        with open('.gitignore', 'w') as f:
            f.write(content)


def main() -> int:
    cli_args = cli.create_cli().parse_args()

    if not cli_args.all_yes:
        code = cli.check_requirements()
        if code != 0:
            return code

    config = cli.get_config(cli_args)

    Term.dev('Project modding started...')

    install_dev_dependencies()

    init_tailwindcss(config.templates_glob)

    generate_files(
        config.live_reload_file,
        config.twcss_file,
        config.minified_twcss_file
    )

    code = update_layout(
        config.root_layout_file,
        config.live_reload_file,
        config.twcss_file,
        config.minified_twcss_file
    )
    if code != 0:
        return code

    if config.gitignore:
        update_gitignore(
            config.static_folder,
            config.twcss_file
        )

    Term.dev(f'Modding complete ✅')
    return 0


if __name__ == "__main__":
    raise SystemExit(main())
